//********************************************************************
//
//   HEADER NAME:
//      GlobalMapperTypes.h - General type and macro definitions.
//   
//********************************************************************

#if !defined _GLOBALMAPPERTYPES_H_
#define      _GLOBALMAPPERTYPES_H_

#ifdef __cplusplus
extern "C" {
#endif

// Make sure the whole thing is packed to 8 byte structure alignment
// like the SDK expects
#pragma pack( push, 8 )

/*--------------------------------------------------------------------
                            LITERAL CONSTANTS
--------------------------------------------------------------------*/

#ifndef PI
#define PI                  3.1415926535898  /* wgs'84 value of pi      */
#endif

#if !defined( NULL )
    #define NULL    0
#endif

#define GM_SUPPORT_EMAIL    _T("support@globalmapper.com")
#define GM_WEB_SITE         _T("http://www.globalmapper.com/")

/*--------------------------------------------------------------------
                                 MACROS
--------------------------------------------------------------------*/

#define DEGTORAD(x) ( (x) * PI / 180.0 )
#define RADTODEG(x) ( (x) * 180.0 / PI )

#define RGB_TO_GRAY(r,g,b) ( ( (b) * 11 + (g) * 59 + (r) * 30 ) / 100 )
#define RGB_TO_GRAY2(c) ( ( (GetBValue(c)) * 11 + (GetGValue(c)) * 59 + (GetRValue(c)) * 30 ) / 100 )

#define     setbit(x)           (1 << x)
#define     cnt_of_array(n)     (sizeof(n)/sizeof(n[0]))
#define     setbits(b,len)      (((1 << len) - 1) << b)
#define     clrbits(b,len)      (~setbits(b,len))
#define     clrbit(b)           (~setbit(b))


#define     getubits(p,b,len)   /* read unsigned from port  */  \
    /*------------------------------------------------------\
    p=port (or var) to get from, b=lowbit, len=#ofbits      \
    ------------------------------------------------------*/\
    ((p >> b) & setbits( 0, len ))


#define     putbits(v,p,b,len)  /* read-modify-write    */  \
    /*------------------------------------------------------\
    v=value, p=port (or var) to set, b=lowbit, len=#ofbits  \
    ------------------------------------------------------*/\
    ((p & clrbits( b, len )) | (((v) << b) & setbits(b,len)))


#define is_bit_set( v, b )      /* is the bit set?      */  \
    ( (v) & ( 1UL << (b) ) )

/*----------------------------------------------------------
Rounds a floating point value to the nearest 32-bit integer
value.
----------------------------------------------------------*/

#define ROUND32( x ) ( (sint32)( ( x < 0.0 ) ? ( x - 0.5 ) : ( x + 0.5 ) ) )
 
/*----------------------------------------------------------
Calculate absolute value, min value, and max value.
----------------------------------------------------------*/

#define ABS_VAL( x )    ( ( (x) < 0 ) ? -(x) : (x) )
#define MAX_VAL( x, y ) ( ( (x) > (y) ) ? (x) : (y) )
#define MIN_VAL( x, y ) ( ( (x) < (y) ) ? (x) : (y) )

/*----------------------------------------------------------
Force value into range
----------------------------------------------------------*/

#define CLAMP_TO_RANGE( val, min, max ) \
    ( ( (val) < (min) ) ? (min) : ( ( (val) > (max) ) ? (max) : (val) ) )

/*----------------------------------------------------------
See if value in range
----------------------------------------------------------*/

#define VAL_IN_RANGE( val, min, max ) \
    ( ( (val) >= (min) ) && ( (val) <= (max) ) )

/*--------------------------------------------------------------------
                                 TYPES
--------------------------------------------------------------------*/

// This section defines a bunch of convenient types to make sure we are
// using values of the size that we want even if we someday move to
// platforms where the size of the int is different. If you already have
// identically-named types defined, simply define GLOBAL_MAPPER_TYPES
// prior to including this file to eliminate this section.

#ifndef GLOBAL_MAPPER_TYPES
#define GLOBAL_MAPPER_TYPES

typedef signed char             sint8;
typedef signed short int        sint16;
typedef signed long int         sint32;
typedef signed int              sint;
typedef unsigned int            uint;
typedef unsigned char           uint8;
typedef unsigned short int      uint16;
typedef unsigned long int       uint32;
typedef __int64                 sint64;
typedef unsigned __int64        uint64;

typedef uint8 boolean;

#ifdef FALSE
#undef FALSE
#endif

#ifdef TRUE
#undef TRUE
#endif

#define FALSE (uint8)0

#define TRUE (uint8)1

#endif // defined GLOBAL_MAPPER_TYPES

// NOTE: These values are saved in the registry so only add new types
//       to the end.
typedef uint8 GM_AreaUnits_t8; enum 
{ 
    GM_AREA_ACRES,
    GM_AREA_HECTARES,
    GM_AREA_SQ_FT,
    GM_AREA_SQ_METERS,
    GM_AREA_SQ_KM,
    GM_AREA_SQ_MILES,

    GM_AREA_NUM_TYPES
};

// NOTE: These values are saved in the registry so only add new types
//       to the end.
typedef uint8 GM_LinearUnits_t8; enum
{
    GM_LINEAR_METRIC,
    GM_LINEAR_NAUTICAL,
    GM_LINEAR_STATUTE,
    GM_LINEAR_CHAINS,
    GM_LINEAR_YARDS,

    GM_LINEAR_NUM_TYPES
};

// NOTE: These values are saved in the registry so only add new types
//       to the end.
typedef uint8 GM_VolumeUnits_t8; enum
{
    GM_VOLUME_CUBIC_FEET,
    GM_VOLUME_CUBIC_METERS,
    GM_VOLUME_CUBIC_YARDS,
    GM_VOLUME_GALLONS,

    GM_VOLUME_NUM_TYPES
};

// NOTE: The brush styles are stored in the registry so only add new
//       syles at the end.
typedef uint16 GM_BrushStyle_t16; enum
{
    GM_BRUSH_NULL,
    GM_BRUSH_SOLID,

    // Hatched brushes
    GM_BRUSH_MIN_HATCH,
    GM_BRUSH_BACK_DIAG_CROSSHATCH = GM_BRUSH_MIN_HATCH,
    GM_BRUSH_CROSSHATCH,
    GM_BRUSH_DIAG_CROSSHATCH,
    GM_BRUSH_FWD_DIAG_CROSSHATCH,
    GM_BRUSH_HORZ_HATCH,
    GM_BRUSH_VERT_HATCH,
    GM_BRUSH_MAX_HATCH = GM_BRUSH_VERT_HATCH,

    // Pattern brushes
    GM_BRUSH_MIN_PATTERN,
    GM_BRUSH_SAND_PATTERN = GM_BRUSH_MIN_PATTERN,
    GM_BRUSH_INT_WATER_PATTERN,
    GM_BRUSH_SALT_PATTERN,
    GM_BRUSH_CORAL_PATTERN,
    GM_BRUSH_BLACK_CROSS_PATTERN,
    GM_BRUSH_BLACK_DOT_PATTERN,
    GM_BRUSH_MAX_PATTERN = GM_BRUSH_BLACK_DOT_PATTERN,

    GM_BRUSH_NUM_STYLES,

    // Custom Symbol support
    GM_BRUSH_FIRST_CUSTOM = 0x100
};

// NOTE: The pen styles are stored in the registry so only add new
//       syles at the end.
typedef uint16 GM_PenStyle_t16; enum
{ 
    GM_PEN_SOLID = 0,
    GM_PEN_DASH,
    GM_PEN_DOT,
    GM_PEN_DASHDOT,
    GM_PEN_DASHDOTDOT,
    GM_PEN_NULL,
        
    // Stylized lines
    GM_PEN_RAILROAD,
    GM_PEN_STRIPED,                 // A solid pen with a single pixel black stripe down the middle

    // Comb/rake/etc. stylized lines
    GM_PEN_FIRST_DIRECTED,
    GM_PEN_LEFT_FACING_LINE = GM_PEN_FIRST_DIRECTED,    // A railroad tie facing to the left of a line
    GM_PEN_RIGHT_FACING_LINE,                           // A railroad tie facing to the right of a line
    GM_PEN_COMB_LONG_TEETH_LEFT,                        // A long railroad tie facing to the left of a line
    GM_PEN_COMB_LONG_TEETH_RIGHT,                       // A long railroad tie facing to the right of a line
    GM_PEN_COMB_WIDE_SPACED_LEFT,                       // Widely spaced railroad ties facing to the left of a line
    GM_PEN_COMB_WIDE_SPACED_RIGHT,                      // Widely spaced railroad ties facing to the right of a line
    GM_PEN_COMB_ALTERNATING_LEFT,                       // Alternating long and short railroad ties to the left of a line
    GM_PEN_COMB_ALTERNATING_RIGHT,                      // Alternating long and short railroad ties to the right of a line
    GM_PEN_COMB_COUPLED_LEFT,                           // Double teeth to the left of a line
    GM_PEN_COMB_COUPLED_RIGHT,                          // Double teeth to the right of a line
    GM_PEN_COMB_TRIPLET_LEFT,                           // Triplet of teeth to the left of a line
    GM_PEN_COMB_TRIPLET_RIGHT,                          // Triplet teeth to the right of a line
    GM_PEN_RAKE_NARROW_SPACED_LEFT,                     // Narrowly space rake of triangles to the left of a line
    GM_PEN_RAKE_NARROW_SPACED_RIGHT,                    // Narrowly space rake of triangles to the right of a line
    GM_PEN_RAKE_LARGE_SPACED_LEFT,                      // Large spaced rake of triangles to the left of a line
    GM_PEN_RAKE_LARGE_SPACED_RIGHT,                     // Large spaced rake of triangles to the right of a line
    GM_PEN_MINUS_SIGNS_LEFT,                            // Minus signs to left of a line
    GM_PEN_MINUS_SIGNS_RIGHT,                           // Minus signs to right of a line
    GM_PEN_PLUS_SIGNS_LEFT,                             // Plus signs to left of a line
    GM_PEN_PLUS_SIGNS_RIGHT,                            // Plus signs to right of a line
    GM_PEN_DOTS_LEFT,                                   // Dots to left of a line
    GM_PEN_DOTS_RIGHT,                                  // Dots to right of a line
    GM_PEN_T_SHAPED_LEFT,                               // T-shape to left of a line
    GM_PEN_T_SHAPED_RIGHT,                              // T-shape to right of a line
    GM_PEN_RECTS_LEFT,                                  // Rectangles to left of a line
    GM_PEN_RECTS_RIGHT,                                 // Rectangles to right of a line
    GM_PEN_SQUARES_LEFT,                                // Squares to left of a line
    GM_PEN_SQUARES_RIGHT,                               // Squares to right of a line
    GM_PEN_HALF_CIRCLE_NARROW_LEFT,                     // Narrow-spaced half circles to the left of a line
    GM_PEN_HALF_CIRCLE_NARROW_RIGHT,                    // Narrow-spaced half circles to the right of a line
    GM_PEN_HALF_CIRCLE_WIDE_LEFT,                       // Widely-spaced half circles to the left of a line
    GM_PEN_HALF_CIRCLE_WIDE_RIGHT,                      // Widely-spaced half circles to the right of a line
    GM_PEN_DIAMONDS_LEFT,                               // Filled diamonds to the left of a line
    GM_PEN_DIAMONDS_RIGHT,                              // Filled diamonds to the right of a line
    GM_PEN_T_SHAPED_LEFT_NO_LINE,                       // T-shape to left of an invisible line
    GM_PEN_T_SHAPED_RIGHT_NO_LINE,                      // T-shape to right of an invisible line
    GM_PEN_LAST_DIRECTED = GM_PEN_T_SHAPED_RIGHT_NO_LINE,

    // Special symbol styles that are directed track of symbols towards one end of the line or the other
    GM_PEN_FIRST_DIRECTED_TRACK = 0x40,
    GM_PEN_V_TRACK_FORWARD = GM_PEN_FIRST_DIRECTED_TRACK,   // Line made up of V symbols opening away the start of the line
    GM_PEN_V_TRACK_REVERSE,                                 // Line made up of V symbols opening towards the start of the line
    GM_PEN_C_TRACK_FORWARD,                                 // Line made up of C symbols opening away from the start of the line
    GM_PEN_C_TRACK_REVERSE,                                 // Line made up of C symbols opening towards from the start of the line
    GM_PEN_ARROW_TRACK_REVERSE,                             // Line made up of triangles pointing towards the start of the line
    GM_PEN_ARROW_TRACK_FORWARD,                             // Line made up of triangles pointing away from the start of the line
    GM_PEN_V_TRACK_SOLID_FORWARD,                           // Line made up of V symbols opening away the start of the solid line
    GM_PEN_V_TRACK_SOLID_REVERSE,                           // Line made up of V symbols opening towards the start of the solid line
    GM_PEN_LAST_DIRECTED_TRACK = GM_PEN_V_TRACK_SOLID_REVERSE,

    // Special symbol styles along solid line
    GM_PEN_FIRST_SOLID_SYMBOL = 0x80,
    GM_PEN_SOLID_WITH_X = GM_PEN_FIRST_SOLID_SYMBOL,// Solid line with X's through them
    GM_PEN_SOLID_DIAMONDS,                          // Solid line with solid diamonds through them
    GM_PEN_SOLID_EMPTY_DIAMONDS,                    // Solid line with empty diamonds through them
    GM_PEN_SOLID_CIRCLES,                           // Solid line with solid circles through them
    GM_PEN_SOLID_EMPTY_CIRCLES,                     // Solid line with empty circles through them
    GM_PEN_LAST_SOLID_SYMBOL = GM_PEN_SOLID_EMPTY_CIRCLES,

    // Styles with special symbols at end
    GM_PEN_FIRST_END_SYMBOL = 0x100,
    GM_PEN_ARROW_TO_START = GM_PEN_FIRST_END_SYMBOL,    // Solid line with an arrow pointing to the start of the line
    GM_PEN_ARROW_TO_END,                                // Solid line with an arrow pointing to the end of the line
    GM_PEN_EMPTY_CIRCLE_AT_START,                       // Solid line with an empty circle at the start of the line
    GM_PEN_EMPTY_CIRCLE_AT_END,                         // Solid line with an empty circle at the end of the line
    GM_PEN_STRIPED_BLACK_EDGES,                         // Solid line with a single pixel black stripe down each edge (doesn't really belong in symbol set, but we didn't space it off from the others when initially defined)
    GM_PEN_ARROW_TO_BOTH_ENDS,                          // Solid line with arrows pointing to both ends
    GM_PEN_ARROW_TO_START_DASHED,                       // Dashed line with an arrow pointing to the start of the line
    GM_PEN_ARROW_TO_END_DASHED,                         // Dashed line with an arrow pointing to the end of the line
    GM_PEN_ARROW_TO_BOTH_ENDS_DASHED,                   // Dashed line with arrows pointing to both ends
    GM_PEN_LAST_END_SYMBOL = GM_PEN_ARROW_TO_BOTH_ENDS_DASHED,

    // Custom dash styles
    GM_PEN_FIRST_CUSTOM_DASH_STYLE = 0x140,
    GM_PEN_DASH_VERY_SHORT_WIDE_SPACED = GM_PEN_FIRST_CUSTOM_DASH_STYLE,
    GM_PEN_DASH_SHORT_WIDE_SPACED,
    GM_PEN_DASH_LONG_WIDE_SPACED,
    GM_PEN_DASH_ULTRA_SHORT_CLOSE_SPACED,
    GM_PEN_LAST_CUSTOM_DASH_STYLE = GM_PEN_DASH_ULTRA_SHORT_CLOSE_SPACED,

    // Additional styles
    GM_PEN_FIRST_ADDITIONAL_STYLE = 0x400,
    GM_PEN_RAILROAD_WIDE = GM_PEN_FIRST_ADDITIONAL_STYLE,   // Railroad - Widely spaced
    GM_PEN_RAILROAD_DOUBLE,                                 // Railroad - Double tracks

    GM_PEN_NUM_STYLES,
    GM_PEN_FIRST_STYLIZED = GM_PEN_RAILROAD,
};

// NOTE: The orientations are stored in the registry so only add new
//       values at the end. Note don't go over 7 bits in length due to how
//       they are stored in font flags.
typedef uint8 GM_LabelPlacement_t8; enum
{
    GM_LABEL_AUTO = 0,                  // automatically place label at best location
    GM_LABEL_CENTER = 1,                // center the label on the point
    GM_LABEL_TOP_RIGHT = 2,             // label is up and to the right of point
    GM_LABEL_RIGHT = 3,                 // label is to the right of the point and vertically centered on it
    GM_LABEL_BOTTOM_RIGHT = 4,          // label is below and to the right of point
    GM_LABEL_BOTTOM = 5,                // label is below the point and horizontally centered on it
    GM_LABEL_BOTTOM_LEFT = 6,           // label is below and to the left of point
    GM_LABEL_LEFT = 7,                  // label is left of the point and vertically centered on it
    GM_LABEL_TOP_LEFT = 8,              // label is above and to the left of point
    GM_LABEL_TOP = 9,                   // label is above the point and horizontally centered on it

    GM_LABEL_NUM_PLACEMENTS,

    // Special label placement values
    GM_LABEL_AT_EDGE = 100,             // label line at edge (i.e. start of first on-screen segment)
};

// Enumeration of different raster/elevation sampling methods available
typedef uint8 GM_SamplingMethod_t8; enum
{
    GM_SAMPLING_NEAREST_NEIGHBOR,       // Nearest neighbor sampling
    GM_SAMPLING_BILINEAR_INTERPOLATION, // Bilinear interpolation
    GM_SAMPLING_BICUBIC_INTERPOLATION,  // Bicubic interpolation
    GM_SAMPLING_BOX_AVG_3X3,            // Box averager 3x3 grid
    GM_SAMPLING_BOX_AVG_4X4,            // Box averager 4x4 grid
    GM_SAMPLING_BOX_AVG_5X5,            // Box averager 5x5 grid
    GM_SAMPLING_BOX_AVG_7X7,            // Box averager 7x7 grid
    GM_SAMPLING_BOX_AVG_9X9,            // Box averager 9x9 grid
    GM_SAMPLING_BOX_MAX_3X3,            // Box maximizer 3x3 grid
    GM_SAMPLING_BOX_MAX_4X4,            // Box maximizer 4x4 grid
    GM_SAMPLING_BOX_MAX_5X5,            // Box maximizer 5x5 grid
    GM_SAMPLING_BOX_MAX_7X7,            // Box maximizer 7x7 grid
    GM_SAMPLING_BOX_MAX_9X9,            // Box maximizer 9x9 grid
    GM_SAMPLING_BOX_MIN_3X3,            // Box minimizer 3x3 grid
    GM_SAMPLING_BOX_MIN_4X4,            // Box minimizer 4x4 grid
    GM_SAMPLING_BOX_MIN_5X5,            // Box minimizer 5x5 grid
    GM_SAMPLING_BOX_MIN_7X7,            // Box minimizer 7x7 grid
    GM_SAMPLING_BOX_MIN_9X9,            // Box minimizer 9x9 grid

    GM_NUM_SAMPLING_METHODS
};

// Enumeration of native languages possible for a build. It is important that
// these values don't change, so only add to the end.
typedef uint8 GM_Language_t8; enum
{
    GM_LANG_ENGLISH = 0,    
    GM_LANG_ARABIC = 1,
    GM_LANG_FRENCH = 2,
    GM_LANG_GREEK = 3,
    GM_LANG_ITALIAN = 4,
    GM_LANG_KOREAN = 5,
    GM_LANG_JAPANESE = 6,
    GM_LANG_PORTUGESE = 7,
    GM_LANG_CHINESE_TRADITIONAL = 8,
    GM_LANG_SPANISH = 9,
    GM_LANG_POLISH = 10,
    GM_LANG_RUSSIAN = 11,

    GM_LANG_NUM_LANGUAGES
};

// NOTE: This structure is also used by the Global Mapper SDK, so new
//       members should only be added to the bottom.
typedef struct
{
    uint32      mSize;          // Size of structure
    const char* mShaderName;    // Shader name
    float       mAmbientLight;  // Ambient light level (Range: 0.0 - 1.0)
    float       mVertExag;      // Vertical exaggeration (Range: 0.1 - 10.0)
    float       mLightAltitude; // Light altitude (Range: 0.0 - 90.0)
    float       mLightAzimuth;  // Light azimuth (Range: 0.0 - 360.0)
    boolean     mHillShading;   // Is hill shading enabled?
    boolean     mWaterEnabled;  // Is Water display enabled?
    uint8       mWaterAlpha;    // Alpha translucency of water (Range: 0 - 255)
    uint8       mWaterColorRed; // Water color - red component
    uint8       mWaterColorGrn; // Water color - green component
    uint8       mWaterColorBlue;// Water color - blue component
    float       mWaterLevel;    // Water level (meters)
    uint8       mShadeDarkness; // Maximum hill shader shadow darkness (0 - 255, 0 is black, 255 is white)
    uint8       mShadeHighlight;// Highlight level (whiteness) to apply for direct sunlight (0 is no additional whiteness, and 255 is completely white)

} GM_VerticalDisplayOptions_t;

// Types of progress reported through GM_ProgressCallbackFunc
typedef uint8 GM_ProgressUpdate_t8; enum 
{ 
    GM_PROGRESS_START,          // progress has started
    GM_PROGRESS_UPDATE_STATUS,  // status string updated
    GM_PROGRESS_UPDATE_PERCENT, // percent complete updated
    GM_PROGRESS_STOP,           // operation has completed
    GM_PROGRESS_RENDER_UPDATE,  // the device context for a draw has been updated

    GM_PROGRESS_NUM_TYPES
};

// Callback for retrieving progress for a long operation in Global Mapper.
// Return FALSE from this if you want to cancel the operation.
typedef boolean (_stdcall *GM_ProgressCallbackFunc)
    ( 
    uint32                  aOperationIdx,  // index of operation this progress applies to (useful if nested long operations or multiple threads)
    GM_ProgressUpdate_t8    aUpdateType,    // what kind of progress update is it
    const char*             aStatus,        // current status string
    uint32                  aPercent        // percent complete
    );

/*--------------------------------------------------------------------
                                MACROS
--------------------------------------------------------------------*/

/*----------------------------------------------------------
The following macros are for converting byte-order from
the native system byte order to little endian byte-order 
(i.e. Intel byte order) or big endian byte-order.
----------------------------------------------------------*/

/* Swaps two values in place */
#define swap_vals( a, b ) { (a) ^= (b); (b) ^= (a); (a) ^= (b); }    

#ifdef IS_BIG_ENDIAN

/* Convert a 16-bit value from host to little endian byte-order */
#define htol16( val ) swap_vals( ((uint8*)( val ))[0], ((uint8*)( val ))[1] )

/* Convert a 16-bit value from little endian to host byte-order */
#define ltoh16( val ) htol16( val )

/* Convert a 32-bit value from host to little endian byte-order */
#define htol32( val )                                           \
    {                                                           \
    swap_vals( ((uint8*)( val ))[0], ((uint8*)( val ))[3] );    \
    swap_vals( ((uint8*)( val ))[1], ((uint8*)( val ))[2] );    \
    }

/* Convert a 32-bit value from little endian to host byte-order */
#define ltoh32( val ) htol32( val )

/* Convert a 64-bit value from host to little endian byte-order */
#define htol64( val )                                           \
    {                                                           \
    swap_vals( ((uint8*)( val ))[0], ((uint8*)( val ))[7] );    \
    swap_vals( ((uint8*)( val ))[1], ((uint8*)( val ))[6] );    \
    swap_vals( ((uint8*)( val ))[2], ((uint8*)( val ))[5] );    \
    swap_vals( ((uint8*)( val ))[3], ((uint8*)( val ))[4] );    \
    }

/* Convert a 64-bit value from little endian to host byte-order */
#define ltoh64( val ) htol64( val )

#else

/* Little endian <--> host conversions */
#define htol16( val )
#define ltoh16( val )
#define htol32( val )
#define ltoh32( val )
#define htol64( val )
#define ltoh64( val )
#define ltoh( val )

/* Big endian <--> host conversions */

/* Convert a 16-bit value from host to big endian byte-order */
#define htob16( val ) swap_vals( ((uint8*)( val ))[0], ((uint8*)( val ))[1] )

/* Convert a 16-bit value from big endian to host byte-order */
#define btoh16( val ) htob16( val )

/* Convert a 32-bit value from host to big endian byte-order */
#define htob32( val )                                           \
    {                                                           \
    swap_vals( ((uint8*)( val ))[0], ((uint8*)( val ))[3] );    \
    swap_vals( ((uint8*)( val ))[1], ((uint8*)( val ))[2] );    \
    }

/* Convert a 32-bit value from big endian to host byte-order */
#define btoh32( val ) htob32( val )

/* Convert a 64-bit value from host to big endian byte-order */
#define htob64( val )                                           \
    {                                                           \
    swap_vals( ((uint8*)( val ))[0], ((uint8*)( val ))[7] );    \
    swap_vals( ((uint8*)( val ))[1], ((uint8*)( val ))[6] );    \
    swap_vals( ((uint8*)( val ))[2], ((uint8*)( val ))[5] );    \
    swap_vals( ((uint8*)( val ))[3], ((uint8*)( val ))[4] );    \
    }

/* Convert a 64-bit value from big endian to host byte-order */
#define btoh64( val ) htob64( val )

#endif

// Restore structure alignment to its old value
#pragma pack( pop )

#ifdef __cplusplus
}
#endif

#endif // end of file GlobalMapperTypes.h
